/***********************************************************************
 *
 Copyright  1995 - 1998, 3Com Corporation or its subsidiaries ("3Com").  
 All rights reserved.
   
 This software may be copied and used solely for developing products for 
 the Palm Computing platform and for archival and backup purposes.  Except 
 for the foregoing, no part of this software may be reproduced or transmitted 
 in any form or by any means or used to make any derivative work (such as 
 translation, transformation or adaptation) without express written consent 
 from 3Com.

 3Com reserves the right to revise this software and to make changes in content 
 from time to time without obligation on the part of 3Com to provide notification 
 of such revision or changes.  
 3COM MAKES NO REPRESENTATIONS OR WARRANTIES THAT THE SOFTWARE IS FREE OF ERRORS 
 OR THAT THE SOFTWARE IS SUITABLE FOR YOUR USE.  THE SOFTWARE IS PROVIDED ON AN 
 "AS IS" BASIS.  3COM MAKES NO WARRANTIES, TERMS OR CONDITIONS, EXPRESS OR IMPLIED, 
 EITHER IN FACT OR BY OPERATION OF LAW, STATUTORY OR OTHERWISE, INCLUDING WARRANTIES, 
 TERMS, OR CONDITIONS OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, AND 
 SATISFACTORY QUALITY.

 TO THE FULL EXTENT ALLOWED BY LAW, 3COM ALSO EXCLUDES FOR ITSELF AND ITS SUPPLIERS 
 ANY LIABILITY, WHETHER BASED IN CONTRACT OR TORT (INCLUDING NEGLIGENCE), FOR 
 DIRECT, INCIDENTAL, CONSEQUENTIAL, INDIRECT, SPECIAL, OR PUNITIVE DAMAGES OF 
 ANY KIND, OR FOR LOSS OF REVENUE OR PROFITS, LOSS OF BUSINESS, LOSS OF INFORMATION 
 OR DATA, OR OTHER FINANCIAL LOSS ARISING OUT OF OR IN CONNECTION WITH THIS SOFTWARE, 
 EVEN IF 3COM HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

 3Com, HotSync, Palm Computing, and Graffiti are registered trademarks, and 
 Palm III and Palm OS are trademarks of 3Com Corporation or its subsidiaries.

 IF THIS SOFTWARE IS PROVIDED ON A COMPACT DISK, THE OTHER SOFTWARE AND 
 DOCUMENTATION ON THE COMPACT DISK ARE SUBJECT TO THE LICENSE AGREEMENT 
 ACCOMPANYING THE COMPACT DISK.

 ********************************************************************************
 *
 * PROJECT: Memo Pad Sample Application
 *
 * FILE: MemoPad.c (Phase 1)
 *
 *       For best formatting set tab size to 3.
 *
 * DESCRIPTION: Sample application to demonstrate the various
 * 	features available.
 *
 *	OBJECTIVE: Create an initial main form with a title and an
 * 	exit button.
 *
 * REVISION HISTORY:
 * 	 9/1/95	David		Initial version
 *
 **********************************************************************/
 
#include <Pilot.h>				// all the system toolbox headers
#include "first_res.h"			// application resource defines
#include <sys_socket.h>
#include <NetMgr.h>

/***********************************************************************
 * Prototypes for internal functions
 **********************************************************************/
static void StartApplication(void);
static Boolean MainFormHandleEvent(EventPtr event);
static void EventLoop(void);
static FieldPtr SetFieldTextFromStr(Word, CharPtr);
static CharPtr getText();
static void appendToTarget(CharPtr);
static void ClearFieldText(Word) ;
static void UpdateScrollBar();
static void PageScroll (DirectionType);
static void Scroll (Short);
static void MyErrorFunc(char *error, char *additional);
static void MyStatusFunc(char *status);
static void transferBuffer(char* bp);
static Boolean MyFormHandleMenuEvent(Word menuID); 
static void ShowConnectionDialog(void);
static void SendName(void);



extern Word AppNetRefnum;

Err errno;
int fd=-1;
CharPtr buffer, bp;
CharPtr server, name;
int port;


/* Converts ascii text to in_addr struct.  NULL is returned if the address
   can not be found. */
static struct in_addr *atoaddr(char *address)
{
  struct hostent *host;
  static struct in_addr saddr;

  /* First try it as aaa.bbb.ccc.ddd. */
  saddr.s_addr = inet_addr(address);
  if (saddr.s_addr != -1) {
    return &saddr;
  }
  host = gethostbyname(address);
  if (host != NULL) {
    return (struct in_addr *) *host->h_addr_list;
  }
  return NULL;
}

/////////////////////////////////////
//read from socket
/////////////////////////////////////


static void readLine(int sockfd) {
	static int bytes_read;
	static char len;
	static Boolean header=true;
	
	AppNetTimeout = SysTicksPerSecond()/2;
	if (header) {
//		bytes_read = read(sockfd, &len, 1);
		bytes_read = NetLibReceive(AppNetRefnum,sockfd,&len,1,NULL,NULL,NULL,50,&errno);
		if (bytes_read == -1) {
		// closed connection
		}
		if (bytes_read == 1) {
			buffer = MemPtrNew(len+1);
			bzero(buffer,len+1);
			bp = buffer;
			header = false;
		}
	}	
	else {
//		bytes_read = read(sockfd,buffer,len);
		bytes_read = NetLibReceive(AppNetRefnum,sockfd,buffer,len,NULL,NULL,NULL,50,&errno);
		if (bytes_read == -1) {
		// closed connection
		}
		if (bytes_read == len) {
			transferBuffer(bp);
			header = true;
			MemPtrFree(bp);
		}
		else {
			buffer += bytes_read;
			len -= bytes_read;
		}
	}
}

static void transferBuffer(char* bp) {
	SetFieldTextFromStr(MainFormTargetField, bp);
}


static int connectToServer(char* server, int port) {

	Word interfaceError;
	Err error;
	struct in_addr *addr;
	int sock, connected;
	struct sockaddr_in address;

	if (SysLibFind( "Net.lib", &AppNetRefnum) == 0) {
		error = NetLibOpen(AppNetRefnum, &interfaceError);	
		if (interfaceError!=0) {
			MyErrorFunc("NetLibOpen error",NULL);
			return -1;
		}

  		addr = atoaddr(server);
  		if (addr == NULL) {
    		MyErrorFunc("make_connection:  Invalid network address.\n", NULL);
    		return -1;
  		}
 
  		memset((char *) &address, 0, sizeof(address));
  		address.sin_family = AF_INET;
  		address.sin_port = (port);
  		address.sin_addr.s_addr = addr->s_addr;

  		sock = socket(AF_INET, SOCK_STREAM, 0);
    	if (sock < 0) {
	  		MyErrorFunc("socket not created.", NULL);
      		return -1;
    	}

    	connected = connect(sock, (struct sockaddr *) &address,sizeof(address));
    	if (connected < 0) {
	  		MyErrorFunc("connect", NULL);
      		return -1;
    	}
    	return sock;
	}
	else
		MyErrorFunc("Can't SysLibFind", NULL);
   		return -1;
}

static void disconnect() {
	if ( fd >= 0 ) close( fd );
	NetLibClose(AppNetRefnum, false);	
}

static void MyErrorFunc(char *error, char *additional)
{
	FrmCustomAlert(ErrorAlert, error, additional ? additional : "", NULL);
}


static void MyStatusFunc(char *status)
{
	FrmCustomAlert(StatusAlert, status, NULL, NULL);
}


/***********************************************************************
 *
 * FUNCTION:     StartApplication
 *
 * DESCRIPTION:  This routine sets up the initial state of the application.
 *
 * PARAMETERS:   None.
 *
 * RETURNED:     Nothing.
 *
 ***********************************************************************/
static void StartApplication(void)
{
	FormPtr	frm;

	// Initialize and draw the main memo pad form.
	frm = FrmInitForm(MainFormForm);	
	FrmSetActiveForm(frm);
	FrmDrawForm(frm);
//	MenuBarPtr mbptr = MenuInit(CreatedforMenuBarIDMenuBar);
	name = MemPtrNew(StrLen("konrad")+1);
	StrCopy(name,"konrad");
	server = MemPtrNew(StrLen("128.230.21.199")+1);
	StrCopy(server,"128.230.21.199");
	port=5678;
	ShowConnectionDialog();
	fd = connectToServer(server,port);
	SendName();
}




static void ShowConnectionDialog() 
{
	FormPtr previousForm = FrmGetActiveForm();
	FormPtr frm = FrmInitForm(NameDialogForm);
	Word hitButton;
	FieldPtr fldP;
	CharPtr txtP;
	char p[5];
	FrmSetActiveForm(frm);
	
	
	SetFieldTextFromStr(nameDialogServerField,server);
	SetFieldTextFromStr(nameDialogNameField,name);
	sprintf(p,"%d",port);
	SetFieldTextFromStr(nameDialogPortField,p);
	
	
	hitButton = FrmDoDialog(frm);
	//read values
	
	fldP = FrmGetObjectPtr(frm,FrmGetObjectIndex(frm,nameDialogServerField));
	txtP = FldGetTextPtr(fldP);
	if (server)
		MemPtrFree(server);
	server = MemPtrNew(StrLen(txtP)+1);
	StrCopy(server,txtP);
	
	fldP = FrmGetObjectPtr(frm,FrmGetObjectIndex(frm,nameDialogNameField));
	txtP = FldGetTextPtr(fldP);
	if (name)
		MemPtrFree(name);
	name = MemPtrNew(StrLen(txtP)+1);
	StrCopy(name,txtP);

	fldP = FrmGetObjectPtr(frm,FrmGetObjectIndex(frm,nameDialogPortField));
	port = atoi(FldGetTextPtr(fldP));

	if (previousForm)
		FrmSetActiveForm(previousForm);
	FrmDeleteForm(frm);
}

static void SendName()
{
  	 	char l = (char)strlen(name);
		write(fd,&l,1);
		write(fd,name,strlen(name));
	
}

/***********************************************************************
 *
 * FUNCTION:    GetObjectPtr
 *
 * DESCRIPTION: P10. This routine returns a pointer to an object in the active form.
 *
 * PARAMETERS:  objectID - id of the object
 *
 * RETURNED:    pointer to the object's data structure
 *
 ***********************************************************************/
static VoidPtr GetObjectPtr(Int objectID)
{
	FormPtr frm;
	
	frm = FrmGetActiveForm();
	return(FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, objectID)));
}

/***********************************************************************
 *
 * FUNCTION:		MainFormHandleEvent
 *
 * DESCRIPTION:	Handles processing of events for the main form.
 *
 * PARAMETERS:		event	- the most recent event.
 *
 * RETURNED:		True if the event is handled, false otherwise.
 *
 ***********************************************************************/
static Boolean MainFormHandleEvent(EventPtr event)
{
	Boolean		handled = false;
	EventType	newEvent;

	switch (event->eType)
	{
	case nilEvent:
//		SetFieldTextFromStr(MainFormTargetField,"dupa");
//		Scroll (5);
//		UpdateScrollBar();
		readLine(fd);
		handled = true;
		break;	


	case menuEvent:
		handled = MyFormHandleMenuEvent(event->data.menu.itemID);
		break;
			
   	case keyDownEvent:  // P13. User used Graffiti or a physical button
   		if (event->data.keyDown.chr == pageUpChr)
				{
				PageScroll (up);
				handled = true;
				}
	
			else if (event->data.keyDown.chr == pageDownChr)
				{
				PageScroll (down);
				handled = true;
				}
	
			break;
   	
			
		case sclRepeatEvent:  // P13. the scroll bar was pressed
			Scroll (event->data.sclRepeat.newValue - 
				event->data.sclRepeat.value);

	   	// Repeating controls don't repeat if handled is set true.
			break;
			
			
   	case ctlSelectEvent:  // A control button was pressed and released.
   		if (event->eType == ctlSelectEvent)
  	 	{
  	 	char l;
		// A control button is pressed and released.
		// The exit button being the only button, stop the application.
//   	MemSet(&newEvent, sizeof(EventType), 0);
//   	newEvent.eType = appStopEvent;
//   	EvtAddEventToQueue(&newEvent);
//		SetFieldTextFromStr(MainFormTargetField,"dupa");
		CharPtr txtP = getText();
		l=(char)strlen(txtP);
		write(fd,&l,1);
		write(fd,txtP,strlen(txtP));
//		ErrNonFatalDisplayIf(i==0,"i=0");
		SetFieldTextFromStr(MainFormTargetField,txtP);
		ClearFieldText(MainFormEditField);
//		appendToTarget(txtP);
		Scroll (5);
		UpdateScrollBar();
		handled = true;
 	 	}
	}
	return(handled);
}


static Boolean MyFormHandleMenuEvent(Word menuID) 
{
	Boolean handled = false;
	EventType	newEvent;
	switch (menuID) {
	case MenuExit:
	   	MemSet(&newEvent, sizeof(EventType), 0);
   		newEvent.eType = appStopEvent;
	   	EvtAddEventToQueue(&newEvent);
		handled=true;
		break;
	case MenuDisconnect:
		disconnect();
		break;
	case MenuConnect:
		disconnect();
		ShowConnectionDialog();
		fd = connectToServer(server,port);
		SendName();
	}
	return handled;
}

static void UpdateScrollBar ()
{
	Word scrollPos;
	Word textHeight;
	Word fieldHeight;
	Short maxValue;
	FieldPtr fld;
	ScrollBarPtr bar;

	fld = GetObjectPtr (MainFormTargetField);
	bar = GetObjectPtr (MainFormScrollbarScrollBar);
	
	// get the values necessary to update the scroll bar.
	FldGetScrollValues (fld, &scrollPos, &textHeight,  &fieldHeight);

	if (textHeight > fieldHeight)
		maxValue = textHeight - fieldHeight;
	else if (scrollPos)
		maxValue = scrollPos;
	else
		maxValue = 0;

	SclSetScrollBar (bar, scrollPos, 0, maxValue, fieldHeight-1);
}

static FieldPtr SetFieldTextFromHandle(Word fieldID, Handle txtH)
{
	Handle oldTxtH;
	FormPtr frm = FrmGetActiveForm();
	FieldPtr fldP;
	CharPtr txtP,ntP;
	
	fldP = FrmGetObjectPtr(frm,FrmGetObjectIndex(frm,fieldID));
	ErrNonFatalDisplayIf(!fldP,"missing field");
	oldTxtH = FldGetTextHandle(fldP);
	//my stuff
	if (oldTxtH) 
	{
		Handle tH;
		CharPtr tP;
		FldSetTextHandle(fldP,NULL);
		//old text
		txtP = MemHandleLock(oldTxtH);
		//new text
		ntP = MemHandleLock(txtH);
		//handle for the result
		tH = MemHandleNew(StrLen(txtP)+StrLen(ntP)+2);
		//pointer to the result
		tP = MemHandleLock(tH);
		//copy
		StrCopy(tP, txtP);
		StrCat(tP,"\n");
		StrCat(tP,ntP);
		//unlock
		MemHandleUnlock(tH);
		MemHandleUnlock(txtH);
		MemHandleUnlock(oldTxtH);
		//set handle and redraw
		FldSetTextHandle(fldP,tH);
		FldDrawField(fldP);
		//free memory
		MemHandleFree(oldTxtH);
		if (txtH)
			MemHandleFree(txtH);
	}
	else
	{
		FldSetTextHandle(fldP,txtH);
		FldDrawField(fldP);
	}
}

static FieldPtr SetFieldTextFromStr(Word fieldID, CharPtr strP)
{
	Handle txtH;
	
	txtH = MemHandleNew(StrLen(strP)+1);
	if (!txtH)
		return NULL;
		
	StrCopy(MemHandleLock(txtH),strP);
	MemHandleUnlock(txtH);
	
	return SetFieldTextFromHandle(fieldID, txtH);
}

static void ClearFieldText(Word fieldID) 
{
	FormPtr frm = FrmGetActiveForm();
	FieldPtr fldP;
	Handle tH;
	
	fldP = FrmGetObjectPtr(frm,FrmGetObjectIndex(frm,fieldID));
	tH = FldGetTextHandle(fldP); 
	FldSetTextHandle(fldP,NULL);
	FldDrawField(fldP);
	if (tH)
		MemHandleFree(tH);
}

static CharPtr getText()
{
	FormPtr frm = FrmGetActiveForm();
	FieldPtr fldP;
//	Handle h;
	CharPtr txtP;
	
	fldP = FrmGetObjectPtr(frm,FrmGetObjectIndex(frm,MainFormEditField));
//	h = FldGetTextHandle(fldP);
	txtP = FldGetTextPtr(fldP);
	return txtP;
}

/***********************************************************************
 *
 * FUNCTION:		EventLoop
 *
 * DESCRIPTION:	A simple loop that obtains events from the Event
 *						Manager and passes them on to various applications and
 *						system event handlers before passing them on to
 *						FrmHandleEvent for default processing.
 *
 * PARAMETERS:		None.
 *
 * RETURNED:		Nothing.
 *
 ***********************************************************************/
static void EventLoop(void)
{
	EventType	event;
	Word error;
	
	do
		{
		// Get the next available event.
		EvtGetEvent(&event, 100);
		
		// Give the system a chance to handle the event.
		if (! SysHandleEvent (&event))
			if (!MenuHandleEvent(0,&event,&error))
			// Give the application a chance to handle the event.
			if (! MainFormHandleEvent(&event))

				// Let the form object provide default handling of the event.
				FrmHandleEvent(FrmGetActiveForm(), &event);
		} 
	while (event.eType != appStopEvent);
								// ** SPECIAL NOTE **
								// In order for the Emulator to exit
								// cleanly when the File|Quit menu option is
								// selected, the running application
								// must exit.  The emulator will generate an 
								// appStopEvent when Quit is selected.
	disconnect();								
}


/***********************************************************************
 *
 * FUNCTION:		PilotMain
 *
 * DESCRIPTION:	This function is the equivalent of a main() function
 *						in standard C.  It is called by the Emulator to begin
 *						execution of this application.
 *
 * PARAMETERS:		cmd - command specifying how to launch the application.
 *						cmdPBP - parameter block for the command.
 *						launchFlags - flags used to configure the launch.			
 *
 * RETURNED:		Any applicable error code.
 *
 ***********************************************************************/
DWord PilotMain(Word cmd, Ptr cmdPBP, Word launchFlags)
{
	// Check for a normal launch.
	if (cmd == sysAppLaunchCmdNormalLaunch)
		{
		// Set up initial form.
		StartApplication();
			
		// Start up the event loop.
		EventLoop();
		}
	
	return(0);
}


/***********************************************************************
 *
 * FUNCTION:    EditViewPageScroll
 *
 * DESCRIPTION: P13. Scrolls the edit view a page at a time.
 *
 * PARAMETERS:   direction - up or down
 *
 * RETURNED:    nothing
 *
 ***********************************************************************/
static void PageScroll (DirectionType direction)
{
	Short value;
	Short min;
	Short max;
	Short pageSize;
	Word linesToScroll;
	FieldPtr fld;
	ScrollBarPtr bar;

	fld = GetObjectPtr (MainFormTargetField);
	
	if (FldScrollable (fld, direction))
		{
		linesToScroll = FldGetVisibleLines (fld) - 1;
		FldScrollField (fld, linesToScroll, direction);

		// Update the scroll bar.
		bar = GetObjectPtr (MainFormScrollbarScrollBar);
		SclGetScrollBar (bar, &value, &min, &max, &pageSize);

		if (direction == up)
			value -= linesToScroll;
		else
			value += linesToScroll;
		
		SclSetScrollBar (bar, value, min, max, pageSize);
		}
}



/***********************************************************************
 *
 * FUNCTION:    EditViewScroll
 *
 * DESCRIPTION: P13.  Scrolls the edit view a line at a time.
 *
 * PARAMETERS:  linesToScroll - number of lines to scroll, 
 *                              negative numbers are used to scroll up
 *
 * RETURNED:    nothing
 *
 ***********************************************************************/
static void Scroll (Short linesToScroll)
{
	Word				blankLines;
	Short				min;
	Short				max;
	Short				value;
	Short				pageSize;
	FieldPtr			fld;
	ScrollBarPtr	bar;
	
	fld = GetObjectPtr (MainFormTargetField);

	if (linesToScroll < 0)
		{
		blankLines = FldGetNumberOfBlankLines (fld);
		FldScrollField (fld, -linesToScroll, up);
		
		// If there were blank lines visible at the end of the field
		// then we need to update the scroll bar.
		if (blankLines)
			{
			// Update the scroll bar.
			bar = GetObjectPtr (MainFormScrollbarScrollBar);
			SclGetScrollBar (bar, &value, &min, &max, &pageSize);
			if (blankLines > -linesToScroll)
				max += linesToScroll;
			else
				max -= blankLines;
			SclSetScrollBar (bar, value, min, max, pageSize);
			}
		}

	else if (linesToScroll > 0)
		FldScrollField (fld, linesToScroll, down);
}



